@@ -59,7 +59,7 @@ class Instance:
             hub, "user_instance", ["user_id", "instance_id"]
         )
 
-    def get_or_create(self, storage_id: str):
+    def insert_if_not_exists(self, storage_id: str):
         instance = self.get()
         if instance is None:
             instance = self.insert(storage_id)
@@ -112,6 +112,13 @@ class Instance:
             }
         )
 
+    def user_count(self, instance_id: str):
+        statement = (
+            self.hub.table("user_instance").select("*").eq("instance_id", instance_id)
+        )
+        data = statement.execute()
+        return len(data.data)
+
 
 class UserInstance:
     def __init__(self, hub: Client) -> None:
@@ -317,12 +324,20 @@ class DtransformIn:
 
     def exists(self, dtransform_id: str, dobject_id: str, dobject_v: str):
         return self.dtransform_in_entity.exists(
-            {"id": dtransform_id, "dobject_id": dobject_id, "dobject_v": dobject_v}
+            {
+                "dtransform_id": dtransform_id,
+                "dobject_id": dobject_id,
+                "dobject_v": dobject_v,
+            }
         )
 
     def delete(self, dtransform_id: str, dobject_id: str, dobject_v: str):
         return self.dtransform_in_entity.delete(
-            {"id": dtransform_id, "dobject_id": dobject_id, "dobject_v": dobject_v}
+            {
+                "dtransform_id": dtransform_id,
+                "dobject_id": dobject_id,
+                "dobject_v": dobject_v,
+            }
         )
 
     def insert(
@@ -346,28 +361,38 @@ class DtransformIn:
 class Storage:
     def __init__(self, hub: Client) -> None:
         self.hub = hub
-        self.storage_entity = Entity(hub, "storage", ["root"])
+        self.storage_entity = Entity(hub, "storage", ["id"])
+
+    def insert_if_not_exists(
+        self, id: str, root: Union[Path, CloudPath, str], region: str, type: str
+    ):
+        storage = self.get(root)
+        if storage is None:
+            storage = self.insert(id=id, root=str(root), region=region, type=type)
+        return storage
+
+    def get(self, root: Union[Path, CloudPath, str]):
+        statement = self.hub.table("storage").select("*").eq("root", str(root))
+        data = statement.execute()
+        if len(data.data) > 0:
+            return data
+        return None
 
     def exists(self, root: Union[Path, CloudPath, str]):
-        return self.storage_entity.exists(
-            {
-                "root": str(root),
-            }
-        )
+        data = self.get(root)
+        if data:
+            return True
+        return False
 
-    def delete(self, root: Union[Path, CloudPath, str]):
+    def delete(self, id: str):
         self.storage_entity.delete(
             {
-                "root": str(root),
+                "id": id,
             }
         )
 
     def insert(
-        self,
-        id: str,
-        root: Union[Path, CloudPath, str],
-        region: str,
-        type: str,
+        self, id: str, root: Union[Path, CloudPath, str], region: str, type: str
     ):
         return self.storage_entity.insert(
             {
@@ -378,6 +403,11 @@ class Storage:
             }
         )
 
+    def instance_count(self, storage_id: str):
+        statement = self.hub.table("instance").select("*").eq("storage_id", storage_id)
+        data = statement.execute()
+        return len(data.data)
+
 
 class Usage:
     def __init__(self, hub: Client) -> None:
